<?php

namespace Drupal\blizz_table_field\Plugin\Field\FieldWidget;

use Drupal\Component\Utility\Xss;
use Drupal\Core\Config\ConfigFactoryInterface;
use Drupal\Core\Field\FieldDefinitionInterface;
use Drupal\Core\Field\FieldItemListInterface;
use Drupal\Core\Field\WidgetBase;
use Drupal\Core\Form\FormStateInterface;
use Drupal\Core\Plugin\ContainerFactoryPluginInterface;
use Symfony\Component\DependencyInjection\ContainerInterface;

/**
 * Plugin implementation of the 'table_widget' widget.
 *
 * @FieldWidget(
 *   id = "table_widget",
 *   label = @Translation("Table widget"),
 *   field_types = {
 *     "table"
 *   }
 * )
 */
class TableWidget extends WidgetBase implements ContainerFactoryPluginInterface {

  /**
   * The configuration object for this widget.
   *
   * @var \Drupal\Core\Config\ImmutableConfig
   */
  protected $config;

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container, array $configuration, $plugin_id, $plugin_definition) {
    return new static(
      $plugin_id,
      $plugin_definition,
      $configuration['field_definition'],
      $configuration['settings'],
      $configuration['third_party_settings'],
      $container->get('config.factory')
    );
  }

  /**
   * TableWidget constructor.
   *
   * @param string $plugin_id
   *   The plugin id.
   * @param mixed $plugin_definition
   *   The plugin definition array.
   * @param \Drupal\Core\Field\FieldDefinitionInterface $field_definition
   *   The field definition object.
   * @param array $settings
   *   The settings array.
   * @param array $third_party_settings
   *   The third party settings array.
   * @param \Drupal\Core\Config\ConfigFactoryInterface $config_factory
   *   Drupal's config factory service.
   */
  public function __construct(
    $plugin_id,
    $plugin_definition,
    FieldDefinitionInterface $field_definition,
    array $settings,
    array $third_party_settings,
    ConfigFactoryInterface $config_factory
  ) {
    parent::__construct($plugin_id, $plugin_definition, $field_definition, $settings, $third_party_settings);
    $this->config = $config_factory->get('blizz_table_field.settings');
  }

  /**
   * {@inheritdoc}
   */
  public static function defaultSettings() {
    return [
      'size' => 60,
      'placeholder' => '',
    ] + parent::defaultSettings();
  }

  /**
   * {@inheritdoc}
   */
  public function settingsForm(array $form, FormStateInterface $form_state) {
    $elements = [];

    $elements['size'] = [
      '#type' => 'number',
      '#title' => $this->t('Size of textfield'),
      '#default_value' => $this->getSetting('size'),
      '#required' => TRUE,
      '#min' => 1,
    ];

    $elements['placeholder'] = [
      '#type' => 'textfield',
      '#title' => $this->t('Placeholder'),
      '#default_value' => $this->getSetting('placeholder'),
      '#description' => $this->t('Text that will be shown inside the field until a value is entered. This hint is usually a sample value or a brief description of the expected format.'),
    ];

    return $elements;
  }

  /**
   * {@inheritdoc}
   */
  public function settingsSummary() {
    $summary = [];

    $placeholder_setting = $this->getSetting('placeholder');
    $summary[] = $this->t('Textfield size: @size', ['@size' => $this->getSetting('size')]);
    if (!empty($placeholder_setting)) {
      $summary[] = $this->t('Placeholder: @placeholder', ['@placeholder' => $placeholder_setting]);
    }

    return $summary;
  }

  /**
   * {@inheritdoc}
   */
  public function formElement(FieldItemListInterface $items, $delta, array $element, array &$form, FormStateInterface $form_state) {
    $title = ($element['#title']) ?? $this->t('Table data');

    $format = Xss::filter($items->getSetting('format'));

    $default['json'] = "[\r\n[\r\n\"Column 1\",\r\n\"Column 2\",\r\n\"Column 3\",\r\n\"Column 4\"\r\n],\r\n[\r\n\"\",\r\n\"\",\r\n\"\",\r\n\"\"\r\n],\r\n[\r\n\"\",\r\n\"\",\r\n\"\",\r\n\"\"\r\n],\r\n[\r\n\"\",\r\n\"\",\r\n\"\",\r\n\"\"\r\n],\r\n[\r\n\"\",\r\n\"\",\r\n\"\",\r\n\"\"\r\n ]\r\n]";
    $default['csv'] = " Column 1, Column 2, Column 3, Column 4 \r\n,,,\r\n,,,\r\n,,,\r\n,,,";

    $element['value'] = $element + [
      '#type' => 'hidden',
      '#description' => $this->t('The serialized table data.'),
      '#size' => $this->getSetting('size'),
      '#placeholder' => $this->getSetting('placeholder'),
      '#weight' => -15,
    ];

    $element['value']['#attached']['library'] = ['blizz_table_field/handsontable-' . $format];
    $element['value']['#default_value'] = (isset($items[$delta]->value)) ? $items[$delta]->value : $default[$format];

    $element['pseudo'] = [
      '#type' => 'item',
      '#title' => $title,
    ];

    $element['table'] = [
      '#markup' => '<div class="rendered-table format-' . $format . '"></div>',
      '#weight' => 15,
    ];

    $description = $this->config->get('formatting_options');

    if (!empty($description)) {
      $element['description'] = [
        '#type' => 'details',
        '#title' => $this->t('Formatting options'),
        '#description' => $description,
        '#weight' => 20,
      ];
    }

    return $element;
  }

}
